#=======================================================================
#	$Id: Memory.pm,v 1.5 2005/11/26 20:16:13 pythontech Exp $
#	In-memory triple store
#	Copyright (C) 2004  Python Technology Limited
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
#	02111-1307, USA.
#=======================================================================
package PythonTech::Triples::Memory;
use strict;
use PythonTech::Triples;

our @ISA = qw(PythonTech::Triples);

#-----------------------------------------------------------------------
#	With the object hash, each key is a resource URI (or literal);
#	the corresponding value is a 2-element list.
#	The first list element is a hash with key being predicate and
#	value being hash obj => 1.
#	The second list element is a hash with key being predicate and
#	value being hash subj => 1 (i.e. reverse mapping).
#	  res => [{pred => {obj => 1}}, {rpred => {subj=>1}}]
#-----------------------------------------------------------------------
sub new {
    my $class = shift;
    my $self = {
	# See above
    };
    bless $self, $class;
    return $self;
}

sub add {
    my($self, $subj,$pred,$obj) = @_;
    unless ($self->{$subj}->[0]->{$pred}->{$obj}) {
	$self->{$subj}->[0]->{$pred}->{$obj} = 1;
	$self->{$obj}->[1]->{$pred}->{$subj} = 1;
    }
}

sub remove {
    my($self, $subj,$pred,$obj) = @_;
    if (exists $self->{$subj}
	&& exists $self->{$subj}->[0]->{$pred}
	&& exists $self->{$subj}->[0]->{$pred}->{$obj}) {
	delete $self->{$subj}->[0]->{$pred}->{$obj};
	delete $self->{$obj}->[1]->{$pred}->{$subj};
    }
}

#-----------------------------------------------------------------------
#	Iterate over matching statements, invoking a callback.
#	When callback returns a true value, exit loop and return it.
#-----------------------------------------------------------------------
sub iterate {
    my($self, $subj,$pred,$obj,$cb) = @_;
    if (defined $subj) {
	return unless $self->{$subj};
	if (defined $pred) {
	    my $oh = $self->{$subj}->[0]->{$pred} or return;
	    if (defined $obj) {
		# spo
		if (defined $oh->{$obj}) {
		    my $res = $cb->($subj,$pred,$obj);
		    return $res if $res;
		}
	    } else {
		# sp?
		foreach my $o (keys %$oh) {
		    my $res = $cb->($subj,$pred,$o);
		    return $res if $res;
		}
	    }
	} elsif (defined $obj) {
	    # s?o
	    while (my($p,$oh) = each %{$self->{$subj}->[0]}) {
		if ($oh->{$obj}) {
		    my $res = $cb->($subj,$p,$obj);
		    return $res if $res;
		}
	    }
	} else {
	    # s??
	    while (my($p,$oh) = each %{$self->{$subj}->[0]}) {
		foreach my $o (keys %$oh) {
		    my $res = $cb->($subj,$p,$o);
		    return $res if $res;
		}
	    }
	}
    } elsif (defined $obj) {
	return unless $self->{$obj};
	if (defined $pred) {
	    # ?po
	    my $sh = $self->{$obj}->[1]->{$pred} or return;
	    foreach my $s (keys %$sh) {
		my $res = $cb->($s,$pred,$obj);
		return $res if $res;
	    }
	} else {
	    # ??o
	    while (my($p,$sh) = each %{$self->{$obj}->[1]}) {
		foreach my $s (keys %$sh) {
		    my $res = $cb->($s,$p,$obj);
		    return $res if $res;
		}
	    }
	}
    } elsif (defined $pred) {
	# ?p?
	while (my($s,$sl) = each %$self) {
	    if (exists $sl->[0]->{$pred}) {
		foreach my $o (keys %{$sl->[0]->{$pred}}) {
		    my $res = $cb->($s,$pred,$o);
		    return $res if $res;
		}
	    }
	}
    } else {
	# ???
	while (my($s,$sl) = each %$self) {
	    while (my($p,$oh) = each %{$sl->[0]}) {
		foreach my $o (keys %$oh) {
		    my $res = $cb->($s,$p,$o);
		    return $res if $res;
		}
	    }
	}
    }
    return;			# All done with no bale-out
}

sub sp2o {
    my($self, $subj,$pred) = @_;
    if (exists $self->{$subj} &&
	exists $self->{$subj}->[0]->{$pred}) {
	my @obj = keys %{$self->{$subj}->[0]->{$pred}};
	return wantarray ? @obj : $obj[0];
    }
    return;
}

sub po2s {
    my($self, $pred,$obj) = @_;
    if (exists $self->{$obj} &&
	exists $self->{$obj}->[1]->{$pred}) {
	my @subj = keys %{$self->{$obj}->[1]->{$pred}};
	return wantarray ? @subj : $subj[0];
    }
    return;
}

sub test {
    my($self, $subj,$pred,$obj) = @_;
    if (exists $self->{$subj} &&
	exists $self->{$subj}->[0]->{$pred} &&
	exists $self->{$subj}->[0]->{$pred}->{$obj}) {
	return 1;
    }
    return;
}

sub commit {}
sub rollback {
    die ref(shift)." cannot rollback - all updates instantaneous";
}

1;
