#=======================================================================
#	$Id: ShellCmd.pm,v 1.2 2005/08/22 12:18:08 pythontech Exp $
#	Command line utility for manipulating triplestore
#	Copyright (C) 2005  Python Technology Limited
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
#	02111-1307, USA.
#=======================================================================
package PythonTech::Triples::ShellCmd;
use strict;

sub new {
    my $class = shift;
    my $self = {@_};
    $self->{'triples'} ||= $self->{'db'};
    die "$class: triples (or db) not configured\n" unless $self->{'triples'};
    bless $self, $class;
    return $self;
}

sub usage {
    my $self = shift;
    my $myname = $self->{'name'};
    print STDERR "Usage: $myname info
       $myname configure <attribute> <value>
       $myname ntriples
       $myname load-ntriples [<filename>]
       $myname list <type>
       $myname show <subj>
       $myname query <subj> <pred> <obj>
       $myname add <subj> <pred> <obj>
       $myname set <subj> <pred> <obj>
       $myname remove <subj> <pred> <obj>
       $myname create <subj> <type> [<pred> <obj>]...
       $myname destroy <subj>
       $myname rename <olduri> <newuri>
"; exit 200;
}

#-----------------------------------------------------------------------
#	Main program
#-----------------------------------------------------------------------
sub main {
    my($self,$myname,@args) = @_;
    ($self->{'name'} = $0) =~ s!.*/!!;
    my $triples = $self->{'triples'};

    # Execute command
    @args >= 1 || $self->usage;
    my $cmd = shift @args;
    if ($cmd eq 'info') {
	@args == 0 || $self->usage;
	print "class: ",ref($triples),"\n";
	if ($triples->isa('PythonTech::Triples::TiedHash')) {
	    # Naughtily peek into internals
	    my $db = $triples->{'db'};
	    print "TiedHash/features: $db->{'f'}\n";
	    print "TiedHash/last_id: $db->{'l'}\n";
	}
    } elsif ($cmd eq 'configure') {
	@args == 2 || $self->usage;
	my($attr,$val) = @args;
	$triples->configure({$attr => $val});
	$triples->commit;
    } elsif ($cmd eq 'ntriples') {
	@args == 0 || $self->usage;
	$triples->write_ntriples();
    } elsif ($cmd eq 'load-ntriples') {
	@args == 0 || @args == 1 || $self->usage;
	if (@args == 0) {
	    $triples->read_ntriples(\*STDIN);
	} else {
	    my $filename = shift @args;
	    open(NT,'<',$filename) || die "Cannot open $filename: $!\n";
	    $triples->read_ntriples(\*NT);
	    close(NT);
	}
	$triples->commit;
    } elsif ($cmd eq 'list') {
	@args == 1 || $self->usage;
	my $type = shift @args;
	foreach my $subj ($triples->po2s('rdf:type', $type)) {
	    print "$subj\n";
	}
    } elsif ($cmd eq 'dump') {
	@args == 0 || $self->usage;
	foreach my $stmt ($triples->query(undef,undef,undef)) {
	    my($subj,$pred,$obj) = @$stmt;
	    if ($obj =~ /^=/) {
		print "$subj $pred ",&shellquote($obj),"\n";
	    } else {
		print "$subj $pred $obj\n";
	    }
	}
    } elsif ($cmd eq 'show') {
	@args == 1 || $self->usage;
	my $subj = shift @args;
	foreach my $pred ($triples->s2p($subj)) {
	    foreach my $obj ($triples->sp2o($subj,$pred)) {
		print "$pred\t$obj\n";
	    }
	}
    } elsif ($cmd eq 'query') {
	@args == 3 || $self->usage;
	my($subj,$pred,$obj) = map {$_ eq '?' ? undef : $_} @args;
	foreach my $res ($triples->query($subj,$pred,$obj)) {
	    print join(" ",@$res),"\n";
	}
    } elsif ($cmd eq 'add') {
	@args == 3 || $self->usage;
	my($subj,$pred,$obj) = @args;
	$triples->add($subj,$pred,$obj);
	$triples->commit;
    } elsif ($cmd eq 'set') {
	@args == 3 || $self->usage;
	my($subj,$pred,$obj) = @args;
	foreach my $o ($triples->sp2o($subj,$pred)) {
	    $triples->remove($subj,$pred,$o);
	}
	$triples->add($subj,$pred,$obj);
	$triples->commit;
    } elsif ($cmd eq 'remove') {
	@args == 3 || $self->usage;
	my($subj,$pred,$obj) = @args;
	my $there = $triples->test($subj,$pred,$obj);
	if ($there) {
	    $triples->remove($subj,$pred,$obj);
	    $triples->commit;
	} else {
	    print STDERR "Was not set: $subj $pred $obj\n";
	}
    } elsif ($cmd eq 'create') { # create uri class [prop value]...
	(@args >= 2 && @args%2==0) || $self->usage;
	my($subj,$class) = splice(@args,0,2);
	$triples->add($subj,'rdf:type',$class);
	while (my($pred,$obj) = splice(@args,0,2)) {
	    $triples->add($subj,$pred,$obj);
	}
	$triples->commit;
    } elsif ($cmd eq 'destroy') {	# destroy uri
	die "$0: 'destroy' method unimplemented\n";
#	(@args == 1) || $self->usage;
#	my($subj) = @args;
#	foreach my $po ($triples->query($subj,undef,undef)) {
#	    $triples->remove($subj,@$po);
#	}
#	foreach my $so ($triples->query(undef,$subj,undef)) {
#	    my($s,$o) = @$so;
#	    $triples->remove($s,$subj,$o);
#	}
#	foreach my $sp ($triples->query(undef,undef,$subj)) {
#	    $triples->remove(@$sp,$subj);
#	}
#	$triples->commit;
    } elsif ($cmd eq 'rename') { # rename olduri newuri
	(@args == 2) || $self->usage;
	my($from,$to) = @args;
	foreach my $po ($triples->query($from,undef,undef)) {
	    $triples->remove($from,@$po);
	    $triples->add($to,@$po);
	}
	foreach my $so ($triples->query(undef,$from,undef)) {
	    my($s,$o) = @$so;
	    $triples->remove($s,$from,$o);
	    $triples->add($s,$to,$o);
	}
	foreach my $sp ($triples->query(undef,undef,$from)) {
	    $triples->remove(@$sp,$from);
	    $triples->add(@$sp,$to);
	}
	$triples->commit;
    } else {
	die "$myname: unknown command `$cmd'\n";
    }
}

sub shellquote {
    my($text) = @_;
    $text =~ s/[\$\"\`\\]/\\$&/g;
    $text =~ s/\r/\${cr}/g;
    $text =~ s/\n/\${nl}/g;
    return '"' . $text . '"';
}

1;
